package com.vaadin.demo.springaddressbook.controls;

import com.vaadin.data.Property;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.demo.springaddressbook.contacteditor.ContactEditor;
import com.vaadin.demo.springaddressbook.contactlist.ContactList;
import com.vaadin.demo.springaddressbook.container.Container;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.TextField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * @author Max Matveev &lt;mail@maxmatveev.com&gt;
 *         Date: 29.10.11
 */
@Component
@Scope("session")
public class Controls extends HorizontalLayout {
    @Autowired
    private ContactList contactList;
    @Autowired
    private ContactEditor contactEditor;
    @Autowired
    private Container addressBookData;

    private Button contactRemovalButton;

    @PostConstruct
    protected void init() {
        setWidth("100%");
        initButtons();
        initFilteringControls();
    }

    private void initButtons() {
        // New item button
        addComponent(new Button("+",
                new Button.ClickListener() {
                    public void buttonClick(Button.ClickEvent event) {
                        // Add new contact "John Doe" as the first item
                        Object id = ((IndexedContainer) contactList
                                .getContainerDataSource()).addItemAt(0);
                        contactList.getItem(id).getItemProperty("First Name")
                                .setValue("John");
                        contactList.getItem(id).getItemProperty("Last Name")
                                .setValue("Doe");

                        // Select the newly added item and scroll to the item
                        contactList.setValue(id);
                        contactList.setCurrentPageFirstItemId(id);
                    }
                }));

        // Remove item button
        contactRemovalButton = new Button("-", new Button.ClickListener() {
            public void buttonClick(Button.ClickEvent event) {
                contactList.removeItem(contactList.getValue());
                contactList.select(null);
            }
        });
        contactRemovalButton.setVisible(false);
        addComponent(contactRemovalButton);
        contactList.addListener(new Property.ValueChangeListener() {
            public void valueChange(Property.ValueChangeEvent event) {
                Object id = contactList.getValue();
                contactEditor.setItemDataSource(id == null ? null : contactList
                        .getItem(id));
                contactRemovalButton.setVisible(id != null);
            }
        });
    }

    private void initFilteringControls() {
        for (final String pn : ContactList.VISIBLE_COLS) {
            final TextField sf = new TextField();
            addComponent(sf);
            sf.setWidth("100%");
            sf.setInputPrompt(pn);
            sf.setImmediate(true);
            setExpandRatio(sf, 1);
            sf.addListener(new Property.ValueChangeListener() {
                public void valueChange(Property.ValueChangeEvent event) {
                    addressBookData.removeContainerFilters(pn);
                    if (sf.toString().length() > 0 && !pn.equals(sf.toString())) {
                        addressBookData.addContainerFilter(pn, sf.toString(),
                                true, false);
                    }
                    getApplication().getMainWindow().showNotification(addressBookData.size() + " matches found");
                }
            });
        }
    }
}
